/******************************************
 * EE563 Example Project 1e
 * by: Derek Molloy
 * based on the DevC++ OpenGL template
 *
 ******************************************/

#include <windows.h>  // Header File For Windows
#include <gl\gl.h>	  // Header File For The OpenGL32 Library
#include <gl\glu.h>	  // Header File For The GLu32 Library
#include <cmath>

#define SPHERE 1 // An identifier for our sphere

HDC			hDC=NULL;  // Private GDI Device Context
HGLRC		hRC=NULL;  // Permanent Rendering Context
HWND		hWnd=NULL; // Holds Our Window Handle
HINSTANCE	hInstance; // Holds The Instance Of The Application

// Function Declarations

LRESULT CALLBACK WndProc (HWND hWnd, UINT message, WPARAM wParam, 
        LPARAM lParam);
void enableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC);
int initGL(GLvoid);
int drawGLScene(float theta);
void disableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC);
void defineGLSphere(GLfloat, GLfloat);

// WinMain - the starting point of our application

int WINAPI WinMain (HINSTANCE hInst, HINSTANCE hPrevInstance, 
    LPSTR lpCmdLine, int iCmdShow)
{
    WNDCLASS wc;
    HGLRC hRC;        
    hInstance = hInst;
    MSG msg;
    BOOL bQuit = FALSE;
    float theta = 0.0f;

    // register window class 
    wc.style = CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor (NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) GetStockObject (BLACK_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "EE553GLExample";
    RegisterClass (&wc);

    // create main window 
    hWnd = CreateWindow ("EE553GLExample", "EE553 Example 1j", 
      WS_CAPTION | WS_POPUPWINDOW | WS_VISIBLE,
      0, 0, 256, 256,
      NULL, NULL, hInstance, NULL);

    // enable OpenGL for the window 
    enableOpenGL (hWnd, &hDC, &hRC);
    initGL();
    
    // program main loop 
    while (!bQuit)
    {
        // check for messages 
        if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
        {
            // handle or dispatch messages 
            if (msg.message == WM_QUIT)
            {
                bQuit = TRUE;
            }
            else
            {
                TranslateMessage (&msg);
                DispatchMessage (&msg);
            }
        }
        else
        {
            drawGLScene(theta);
            SwapBuffers (hDC);
            theta += 1.0f;
            Sleep (1);
        }
    }
    // shutdown OpenGL
    disableOpenGL (hWnd, hDC, hRC);
    // destroy the window explicitly 
    DestroyWindow (hWnd);
    return msg.wParam;
}


// Window Callback Process

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_CREATE:
            return 0;
        case WM_CLOSE:
            PostQuitMessage (0);
            return 0;
        case WM_DESTROY:
            return 0;
        case WM_KEYDOWN:
            switch (wParam)
            {
            case VK_ESCAPE:
                PostQuitMessage(0);
                return 0;
            }
            return 0;
        default:
            return DefWindowProc (hWnd, message, wParam, lParam);
    }
}


//Enable OpenGL

void enableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC)
{
    PIXELFORMATDESCRIPTOR pfd;
    int iFormat;

    // get the device context (DC) 
    *hDC = GetDC (hWnd);

    // set the pixel format for the DC 
    ZeroMemory (&pfd, sizeof (pfd));
    pfd.nSize = sizeof (pfd);
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pfd.iPixelType = PFD_TYPE_RGBA;
    pfd.cColorBits = 24;
    pfd.cDepthBits = 16;
    pfd.iLayerType = PFD_MAIN_PLANE;
    iFormat = ChoosePixelFormat (*hDC, &pfd);
    SetPixelFormat (*hDC, iFormat, &pfd);

    // create and enable the render context (RC) 
    *hRC = wglCreateContext( *hDC );
    wglMakeCurrent( *hDC, *hRC );
}

// Setup our GL Scene
int initGL(GLvoid)										
{
	glShadeModel(GL_SMOOTH);            // Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);// White Background
	glClearDepth(1.0f);					// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);			// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);				// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
         	// Really Nice Perspective Calculations

    GLfloat light_pos[] = {-1.0f, 1.0f, 2.0f, 1.0f};
    GLfloat light_Ka[]  = { 0.0f, 0.0f, 0.0f, 1.0f};
    GLfloat light_Kd[]  = { 1.0f, 1.0f, 1.0f, 1.0f};
    GLfloat light_Ks[]  = { 1.0f, 1.0f, 1.0f, 1.0f};
  
    glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
    glLightfv(GL_LIGHT0, GL_AMBIENT, light_Ka);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_Kd);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_Ks);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
  
    // Ambient Lighting
    GLfloat ambient_light[] = {0.75, 0.75, 0.75, 1.0}; //slight ambient light
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient_light);
    
    glEnable(GL_DEPTH_TEST);
   
    defineGLSphere(0.5f, 15.0f);       // call our draw sphere function
    
	return TRUE;					
}

// Called to update the scene - give us the animation
int drawGLScene(float theta)			// Draw the scene; 
{                                       // theta is amount to rotate
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // Clear Screen 
    //glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); // display all mesh lines of the sphere
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0, 1.0, 1.0, 20.0);
    
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(0.0, 0.0, 3.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
    
    GLfloat mat_ambientb[] = { 0.2, 0.0, 0.2, 1.0 };
    GLfloat mat_diffuseb[] = { 0.5, 0.5, 0.5, 1.0 };
    GLfloat mat_specularb[] = { 0.3, 0.3, 0.5, 1.0 };
    GLfloat mat_shininessb[] = { 10.0 };
    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specularb);
    glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininessb);
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambientb);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuseb);
    
    glRotatef (theta, 1.0f, 1.0f, 0.0f);    // Rotate theta around the xy-axis
    glCallList(SPHERE);           
  
    GLfloat mat_ambientg[] = { 0.2, 0.0, 0.2, 1.0 };
    GLfloat mat_diffuseg[] = { 0.5, 0.5, 0.5, 1.0 };
    GLfloat mat_specularg[] = { 0.2, 0.5, 0.2, 1.0 };
    GLfloat mat_shininessg[] = { 10.0 };
    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specularg);
    glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininessg);
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambientg);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuseg);
    
   // glColor3f(0.0f, 0.0f, 1.0f);  // blue
    glTranslatef(1.0f, 0.0f, 0.0f);
    glCallList(SPHERE);
    
    GLfloat mat_ambient[] = { 0.2, 0.0, 0.2, 1.0 };
    GLfloat mat_diffuse[] = { 0.5, 0.5, 0.5, 1.0 };
    GLfloat mat_specular[] = { 0.5, 0.2, 0.2, 1.0 };
    GLfloat mat_shininess[] = { 10.0 };
    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
    //glColor3f(1.0f, 0.0f, 0.0f);  // red
    glTranslatef(-2.0f, 0.0f, 0.0f);
    glCallList(SPHERE);
    
   	return TRUE;   // Keep Going
}

void defineGLSphere(GLfloat radius, GLfloat step)
{
     glNewList(SPHERE, GL_COMPILE);
       GLfloat x,y,z, c = 3.14159f/180.0f;
       for (GLfloat phi=-80.0f; phi<80.0; phi+=step)
       {
           GLfloat phi_rad = c * phi;
           GLfloat phi_rad_end = c * (phi+step);
           glBegin(GL_QUAD_STRIP);
           for (GLfloat theta=-180.0f; theta<=180.0f; theta+=step)
           {
               GLfloat theta_rad = c * theta;
               x = radius * sin(theta_rad)*cos(phi_rad);
               y = radius * cos(theta_rad)*cos(phi_rad);
               z = radius * sin(phi_rad);
               glVertex3f(x,y,z);
               x = radius * sin(theta_rad)*cos(phi_rad_end);
               y = radius * cos(theta_rad)*cos(phi_rad_end);
               z = radius * sin(phi_rad_end);
               glVertex3f(x,y,z); 
           }
           glEnd();
       }    
       
       // Close one end
       
       GLfloat closeRing_rad = c * 80.0;
       glBegin(GL_TRIANGLE_FAN);
         glVertex3f(0.0f, 0.0f, -radius);
         z = radius * -sin(closeRing_rad);
         for(GLfloat theta=-180.0f; theta<=180.0f; theta+=step)
         {
              GLfloat theta_rad = c * theta;
              x = radius * sin(theta_rad) * cos(closeRing_rad);
              y = radius * cos(theta_rad) * cos(closeRing_rad);
              glVertex3f(x,y,z);
         }
       glEnd();
       
       // Close the other
       
       glBegin(GL_TRIANGLE_FAN);
         glVertex3f(0.0f, 0.0f, radius);
         z = radius * sin(closeRing_rad);
         for(GLfloat theta=-180.0f; theta<=180.0f; theta+=step)
         {
              GLfloat theta_rad = c * theta;
              x = radius * sin(theta_rad) * cos(closeRing_rad);
              y = radius * cos(theta_rad) * cos(closeRing_rad);
              glVertex3f(x,y,z);
         }
       glEnd();
     glEndList();
}


// Disable OpenGL 
void disableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC)
{
    wglMakeCurrent (NULL, NULL);
    wglDeleteContext (hRC);
    ReleaseDC (hWnd, hDC);
}
