/******************************************
 * EE563 Example Project 1e
 * by: Derek Molloy
 * based on the DevC++ OpenGL template
 *
 ******************************************/

#include <windows.h>  // Header File For Windows
#include <gl\gl.h>	  // Header File For The OpenGL32 Library
#include <gl\glu.h>	  // Header File For The GLu32 Library
#include <cmath>

#define SPHERE 1 // An identifier for our sphere

HDC			hDC=NULL;  // Private GDI Device Context
HGLRC		hRC=NULL;  // Permanent Rendering Context
HWND		hWnd=NULL; // Holds Our Window Handle
HINSTANCE	hInstance; // Holds The Instance Of The Application

// Function Declarations

LRESULT CALLBACK WndProc (HWND hWnd, UINT message, WPARAM wParam, 
        LPARAM lParam);
void enableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC);
int initGL(GLvoid);
int drawGLScene(float theta);
void disableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC);
void drawGLIcosahedron();
void normalize(float v[3]);
void normcrossprod(float v1[3], float v2[3], float out[3]);


// WinMain - the starting point of our application

int WINAPI WinMain (HINSTANCE hInst, HINSTANCE hPrevInstance, 
    LPSTR lpCmdLine, int iCmdShow)
{
    WNDCLASS wc;
    HGLRC hRC;        
    hInstance = hInst;
    MSG msg;
    BOOL bQuit = FALSE;
    float theta = 0.0f;

    // register window class 
    wc.style = CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor (NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) GetStockObject (BLACK_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "EE553GLExample";
    RegisterClass (&wc);

    // create main window 
    hWnd = CreateWindow ("EE553GLExample", "EE553 Example 1k", 
      WS_CAPTION | WS_POPUPWINDOW | WS_VISIBLE,
      0, 0, 256, 256,
      NULL, NULL, hInstance, NULL);

    // enable OpenGL for the window 
    enableOpenGL (hWnd, &hDC, &hRC);
    initGL();
    
    // program main loop 
    while (!bQuit)
    {
        // check for messages 
        if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
        {
            // handle or dispatch messages 
            if (msg.message == WM_QUIT)
            {
                bQuit = TRUE;
            }
            else
            {
                TranslateMessage (&msg);
                DispatchMessage (&msg);
            }
        }
        else
        {
            drawGLScene(theta);
            SwapBuffers (hDC);
            theta += 0.25f;
            Sleep (1);
        }
    }
    // shutdown OpenGL
    disableOpenGL (hWnd, hDC, hRC);
    // destroy the window explicitly 
    DestroyWindow (hWnd);
    return msg.wParam;
}


// Window Callback Process

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_CREATE:
            return 0;
        case WM_CLOSE:
            PostQuitMessage (0);
            return 0;
        case WM_DESTROY:
            return 0;
        case WM_KEYDOWN:
            switch (wParam)
            {
            case VK_ESCAPE:
                PostQuitMessage(0);
                return 0;
            }
            return 0;
        default:
            return DefWindowProc (hWnd, message, wParam, lParam);
    }
}


//Enable OpenGL

void enableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC)
{
    PIXELFORMATDESCRIPTOR pfd;
    int iFormat;

    // get the device context (DC) 
    *hDC = GetDC (hWnd);

    // set the pixel format for the DC 
    ZeroMemory (&pfd, sizeof (pfd));
    pfd.nSize = sizeof (pfd);
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pfd.iPixelType = PFD_TYPE_RGBA;
    pfd.cColorBits = 24;
    pfd.cDepthBits = 16;
    pfd.iLayerType = PFD_MAIN_PLANE;
    iFormat = ChoosePixelFormat (*hDC, &pfd);
    SetPixelFormat (*hDC, iFormat, &pfd);

    // create and enable the render context (RC) 
    *hRC = wglCreateContext( *hDC );
    wglMakeCurrent( *hDC, *hRC );
}

// Setup our GL Scene
int initGL(GLvoid)										
{
	glShadeModel(GL_FLAT);                
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);// White Background
	glClearDepth(1.0f);					// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);			// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);				// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
         	// Really Nice Perspective Calculations

    GLfloat light_pos[] = {-2.5f, 1.5f, -3.0f, 1.0f};
    GLfloat light_Ka[]  = { 0.0f, 0.0f, 0.0f, 1.0f};
    GLfloat light_Kd[]  = { 1.0f, 1.0f, 1.0f, 1.0f};
    GLfloat light_Ks[]  = { 1.0f, 1.0f, 1.0f, 1.0f};
  
    glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
    glLightfv(GL_LIGHT0, GL_AMBIENT, light_Ka);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_Kd);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_Ks);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
  
    // Ambient Lighting
    GLfloat ambient_light[] = {0.75, 0.75, 0.75, 1.0}; //slight ambient light
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient_light);
    glEnable(GL_DEPTH_TEST);
    
	return TRUE;					
}

// Called to update the scene - give us the animation
int drawGLScene(float theta)			// Draw the scene; 
{                                       // theta is amount to rotate
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // Clear Screen 
    //glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); // display all mesh lines of the sphere

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0, 1.0, 1.0, 20.0);
    
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(0.0, 0.0, 3.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
    
    glPushMatrix();  
      glRotatef (theta, 1.0f, 0.0f, 0.0f);
      drawGLIcosahedron();       // call our draw sphere function         
    glPopMatrix();
    
    GLfloat mat_ambientb[] = { 0.2, 0.0, 0.2, 1.0 };
    GLfloat mat_diffuseb[] = { 0.5, 0.5, 0.5, 1.0 };
    GLfloat mat_specularb[] = { 0.3, 0.3, 0.5, 1.0 };
    GLfloat mat_shininessb[] = { 10.0 };
    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specularb);
    glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininessb);
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambientb);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuseb);
    
   	return TRUE;   // Keep Going
}

void drawGLIcosahedron()   // From the red book
{
     GLfloat x,y,z, c = 3.14159f/180.0f;
     #define X .525731112119133606
     #define Z .850650808352039932
     
     static GLfloat vdata[12][3] = {
            {-X, 0.0, Z}, {X, 0.0, Z}, {-X, 0.0, -Z}, {X, 0.0, -Z},
            {0.0, Z, X}, {0.0, Z, -X}, {0.0, -Z, X}, {0.0, -Z, -X},
            {Z, X, 0.0}, {-Z, X, 0.0}, {Z, -X, 0.0}, {-Z, -X, 0.0}
        };
     static GLuint tindices[20][3] = {
            {0,4,1}, {0,9,4}, {9,5,4}, {4,5,8}, {4,8,1},
            {8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3},
            {7,10,3}, {7,6,10}, {7,11,6}, {11,0,6}, {0,1,6},
            {6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11} };
     int i;
     glBegin(GL_TRIANGLES);
        for (i = 0; i < 20; i++) {
            
            GLfloat d1[3], d2[3], norm[3];
            for (int j = 0; j < 3; j++) {
                d1[j] = vdata[tindices[i][0]][j] - vdata[tindices[i][1]][j];
                d2[j] = vdata[tindices[i][1]][j] - vdata[tindices[i][2]][j];
            }
            normcrossprod(d1, d2, norm);
            glNormal3fv(norm);

            glVertex3fv(&vdata[tindices[i][0]][0]);
            glVertex3fv(&vdata[tindices[i][1]][0]);
            glVertex3fv(&vdata[tindices[i][2]][0]);
        }
     glEnd();
}

void normalize(float v[3]) {
    GLfloat d = sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
    if (d == 0.0) {
        return;
    }
    v[0] /= d; v[1] /= d; v[2] /= d;
}

void normcrossprod(float v1[3], float v2[3], float out[3])
{
    GLint i, j;
    GLfloat length;
    out[0] = v1[1]*v2[2] - v1[2]*v2[1];
    out[1] = v1[2]*v2[0] - v1[0]*v2[2];
    out[2] = v1[0]*v2[1] - v1[1]*v2[0];
    normalize(out);
}

// Disable OpenGL 
void disableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC)
{
    wglMakeCurrent (NULL, NULL);
    wglDeleteContext (hRC);
    ReleaseDC (hWnd, hDC);
}
