/******************************************
 * EE563 Example Project 6 - Scene Graph Example with Graphics
 * by: Derek Molloy
 ******************************************/

#include <windows.h>  // Header File For Windows
#include <gl\gl.h>	  // Header File For The OpenGL32 Library
#include <gl\glu.h>	  // Header File For The GLu32 Library
#include "vec3.h"
#include "Matrix.h"
#include "vec4.h"
#include "Quat.h"
#include "SceneGraph.h"
#include "DummyObject.h"
#include "Primitive.h"
#include "Transform.h"
#include "Light.h"
#include <iostream>
using namespace std;


HDC			hDC=NULL;  // Private GDI Device Context
HGLRC		hRC=NULL;  // Permanent Rendering Context
HWND		hWnd=NULL; // Holds Our Window Handle
HINSTANCE	hInstance; // Holds The Instance Of The Application
GLfloat     fov = 60.0f;
int         downX = 0, downY = 0;
float       xOffset = 0.0f, yOffset = 0.0f;
SceneGraph  *sg;

// Function Declarations

LRESULT CALLBACK WndProc (HWND hWnd, UINT message, WPARAM wParam, 
        LPARAM lParam);
void enableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC);
int initGL(GLvoid);
int drawGLScene(float theta);
void disableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC);
void defineGLSphere(GLfloat, GLfloat);
void increaseFOV();
void decreaseFOV();
void setFOV(GLfloat);

// WinMain - the starting point of our application

int WINAPI WinMain (HINSTANCE hInst, HINSTANCE hPrevInstance, 
    LPSTR lpCmdLine, int iCmdShow)
{
    WNDCLASS wc;
    HGLRC hRC;        
    hInstance = hInst;
    MSG msg;
    BOOL bQuit = FALSE;
    float theta = 0.0f;

    // register window class 
    wc.style = CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor (NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) GetStockObject (BLACK_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "EE553GLExample";
    RegisterClass (&wc);

    // create main window 
    hWnd = CreateWindow ("EE553GLExample", "EE553 Example 6", 
      WS_CAPTION | WS_POPUPWINDOW | WS_VISIBLE,
      0, 0, 256, 256,
      NULL, NULL, hInstance, NULL);

    // enable OpenGL for the window 
    enableOpenGL (hWnd, &hDC, &hRC);
    initGL();
    
    // program main loop 
    while (!bQuit)
    {
        // check for messages 
        if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
        {
            // handle or dispatch messages 
            if (msg.message == WM_QUIT)
            {
                bQuit = TRUE;
            }
            else
            {
                TranslateMessage (&msg);
                DispatchMessage (&msg);
            }
        }
        else
        {
            drawGLScene(theta);
            SwapBuffers (hDC);
            theta += 1.0f;
            Sleep (1);
        }
    }
    // shutdown OpenGL
    disableOpenGL (hWnd, hDC, hRC);
    // destroy the window explicitly 
    DestroyWindow (hWnd);
    return msg.wParam;
}


// Window Callback Process

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_RBUTTONDOWN:  // if the right button is down
             setFOV(60);
             xOffset=0.0f;
             yOffset=0.0f;
             return 0;  
        case WM_LBUTTONDOWN:
             downX = (int)(short)LOWORD(lParam); 
             downY = (int)(short)HIWORD(lParam); 
        case WM_MOUSEMOVE:    // if the mouse is moved
             if (wParam == MK_LBUTTON) // and the left mouse button is down
             {
                int xPos = (int)(short)LOWORD(lParam); 
                int yPos = (int)(short)HIWORD(lParam); 
                int diffx = xPos - downX;
                int diffy = yPos - downY;
                xOffset+= ((float)diffx)/10;
                yOffset+= ((float)diffy)/10;
             }
             return 0;
        case WM_CREATE:
            return 0;
        case WM_CLOSE:
            PostQuitMessage (0);
            return 0;
        case WM_DESTROY:
            return 0;
        case WM_KEYDOWN:
            switch (wParam)
            {
            case VK_ESCAPE:
                 PostQuitMessage(0);
                 return 0;
            case VK_LEFT:
                 xOffset-=0.1f;
                 return 0;
            case VK_RIGHT:
                 xOffset+=0.1f;
                 return 0;
            case VK_UP:
                 yOffset-=0.1f;
                 return 0;
            case VK_DOWN:
                 yOffset+=0.1f;
                 return 0;
            case VK_F1: //F1 key pressed
                 return 0;
            }
            return 0;
        case WM_CHAR:
             switch(wParam)
             {
               case 'a': 
                  decreaseFOV();
                  return 0;        
               case 's':
                  increaseFOV();
                  return 0;      
             }
             return 0;
        default:
            return DefWindowProc (hWnd, message, wParam, lParam);
    }
}


//Enable OpenGL

void enableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC)
{
    PIXELFORMATDESCRIPTOR pfd;
    int iFormat;
    // get the device context (DC) 
    *hDC = GetDC (hWnd);

    // set the pixel format for the DC 
    ZeroMemory (&pfd, sizeof (pfd));
    pfd.nSize = sizeof (pfd);
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pfd.iPixelType = PFD_TYPE_RGBA;
    pfd.cColorBits = 24;
    pfd.cDepthBits = 16;
    pfd.iLayerType = PFD_MAIN_PLANE;
    iFormat = ChoosePixelFormat (*hDC, &pfd);
    SetPixelFormat (*hDC, iFormat, &pfd);

    // create and enable the render context (RC) 
    *hRC = wglCreateContext( *hDC );
    wglMakeCurrent( *hDC, *hRC );
}

// Setup our GL Scene
int initGL(GLvoid)										
{
	glShadeModel(GL_SMOOTH);            // Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);// Black Background
	glClearDepth(1.0f);					// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);			// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);				// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	
                                   // Really Nice Perspective Calculations
         
    sg = new SceneGraph;
    DummyObject *root     = new DummyObject("Root");
    sg->bindRoot(root);
    
    Light *l = new Light();
    root->addChild(l);
    
    Primitive *p = new Primitive(SPHERE, 1.0f, 36.0f);
    p->getMaterial()->setDiffuse(Vec4(0.18f, 0.81f, 0.31f, 1.0f));  //green
    root->addChild(p);
    
    Matrix m1, m2, m3;
    m1.makeTranslate(2.0f, 0.0f, 0.0f);
    Transform *t1 = new Transform(m1);
    root->addChild(t1);
    Primitive *p1 = new Primitive(SPHERE, 2.5f, 20.0f);  //blue by default
    t1->addChild(p1);
    
    m2.makeTranslate(-2.0f, 0.0f, 0.0f);
    Transform *t2 = new Transform(m2);
    root->addChild(t2);
    Primitive *p2 = new Primitive(SPHERE, 2.5f, 10.0f);
    p2->getMaterial()->setDiffuse(Vec4(0.89f, 0.25f, 0.14f, 1.0f));  //red
    t2->addChild(p2);
    
    m3.makeTranslate(0.0f, 1.25f, 0.0f);
    Transform *t3  = new Transform(m3);
    p2->addChild(t3);
    Primitive *p3 = new Primitive(SPHERE, 0.5f, 10.0f);
    p3->getMaterial()->setDiffuse(Vec4(0.89f, 0.25f, 0.14f, 1.0f));  //red
    t3->addChild(p3);
        
	return TRUE;						// Initialization Went OK
}

void increaseFOV() {    
     if (fov<180.0f) fov+=1.0;
}

void decreaseFOV(){
     if (fov>0.0f) fov-=1.0;
}

void setFOV(GLfloat f){
     if (f>0.0f && f<180.0f) fov = f;
}

// Called to update the scene - give us the animation
int drawGLScene(float theta)			// Draw the scene; 
{                                       // theta is amount to rotate
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // Clear Screen 
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); // display all mesh lines of the sphere
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(fov, 1.0, 0.1, 100.0);
    
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(0.0, 0.0, 10.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
    glTranslatef(xOffset, yOffset, 0);

    glColor3f(0.0f, 1.0f, 0.0f);  // green
    sg->renderScene(0.0f);
    
	return TRUE;   // Keep Going
}

// Disable OpenGL 
void disableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC)
{
    wglMakeCurrent (NULL, NULL);
    wglDeleteContext (hRC);
    ReleaseDC (hWnd, hDC);
}
